#!/usr/bin/env node
"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.babelDualPackage = void 0;
var _nodeProcess = require("node:process");
var _nodePerf_hooks = require("node:perf_hooks");
var _nodePath = require("node:path");
var _promises = require("node:fs/promises");
var _nodeFs = require("node:fs");
var _core = require("@babel/core");
var _glob = require("glob");
var _init = require("./init.cjs");
var _transform = require("./transform.cjs");
var _util = require("./util.cjs");
const babelDualPackage = async moduleArgs => {
  const ctx = await (0, _init.init)(moduleArgs, _util.logError);
  if (ctx) {
    const {
      args,
      babelProjectConfig
    } = ctx;
    const {
      targets,
      plugins,
      presets,
      babelrc: unused,
      filename: overwritten,
      ...options
    } = babelProjectConfig.options;
    const outDir = (0, _nodePath.resolve)((0, _nodePath.relative)((0, _nodeProcess.cwd)(), args.values['out-dir']));
    const cjsOutDir = (0, _nodePath.join)(outDir, args.values['cjs-dir-name']);
    const keepFileExtension = args.values['keep-file-extension'];
    const outFileExtension = args.values['out-file-extension'];
    const noCjsDir = args.values['no-cjs-dir'];
    const minified = args.values.minified || (options.minified ?? false);
    const noComments = args.values['no-comments'] || (Object.hasOwn(options, 'comments') ? !options.comments : false);
    const sourceMaps = args.values['source-maps'] || (options.sourceMaps ?? false);
    const copyFiles = args.values['copy-files'];
    const {
      extensions
    } = args.values;
    if (!presets.length) {
      (0, _util.addDefaultPresets)(presets, extensions);
    }
    let numFilesCompiled = 0;
    let compileTime = 0;
    let [tsFilesUpdated, tsUpdateTime, tsStartTime] = [0, 0, 0];
    const dmctsRegex = /\.d\.[mc]?ts$/;
    const esmPresets = (0, _util.getModulePresets)(presets, 'esm');
    const cjsPresets = (0, _util.getModulePresets)(presets, 'cjs');
    const esmPlugins = (0, _util.getEsmPlugins)(plugins);
    const startTime = _nodePerf_hooks.performance.now();
    const build = async (filename, positional) => {
      const {
        code,
        map
      } = await (0, _transform.transform)(filename, {
        ...options,
        targets,
        presets,
        plugins,
        filename,
        minified,
        sourceMaps,
        ast: false,
        sourceType: 'module',
        comments: !noComments,
        esmPresets,
        esmPlugins,
        cjsPresets,
        outFileExtension,
        keepFileExtension
      });
      if (code) {
        const extRegex = /(\.\w+)$/i;
        const relativeFn = positional ? (0, _nodePath.relative)(positional, filename) : (0, _nodePath.basename)(filename);
        const esmExt = (0, _util.getOutExt)(filename, outFileExtension, keepFileExtension);
        const cjsExt = (0, _util.getOutExt)(filename, outFileExtension, keepFileExtension, 'cjs');
        let outEsm = (0, _nodePath.join)(outDir, relativeFn);
        let outCjs = (0, _nodePath.join)(noCjsDir ? outDir : cjsOutDir, relativeFn);
        outEsm = outEsm.replace(extRegex, esmExt);
        outCjs = outCjs.replace(extRegex, cjsExt);
        await (0, _promises.mkdir)((0, _nodePath.dirname)(outEsm), {
          recursive: true
        });
        await (0, _promises.mkdir)((0, _nodePath.dirname)(outCjs), {
          recursive: true
        });
        await (0, _promises.writeFile)(outEsm, code.esm);
        await (0, _promises.writeFile)(outCjs, code.cjs);
        if (sourceMaps && map) {
          await (0, _promises.writeFile)(`${outEsm}.map`, JSON.stringify(map.esm, null, 2));
          await (0, _promises.writeFile)(`${outCjs}.map`, JSON.stringify(map.cjs, null, 2));
        }
      }
    };
    for (const pos of args.positionals) {
      const posPath = (0, _nodePath.resolve)((0, _nodePath.relative)((0, _nodeProcess.cwd)(), pos));
      let stats = null;
      try {
        stats = await (0, _promises.stat)(posPath);
      } catch {
        continue;
      }
      if (stats.isFile()) {
        const fileHandling = await (0, _util.getBabelFileHandling)(posPath);
        if (!dmctsRegex.test(posPath) && extensions.includes((0, _nodePath.extname)(posPath)) && fileHandling === 'transpile') {
          await build(posPath);
          numFilesCompiled++;
        }
      } else {
        const allFiles = await (0, _util.getFiles)(posPath);
        const fileHasExpectedExt = file => extensions.includes((0, _nodePath.extname)(file));
        const files = allFiles.filter(file => fileHasExpectedExt(file) && !dmctsRegex.test(file));
        const copyable = !copyFiles ? [] : allFiles.filter(file => !fileHasExpectedExt(file) && !/^(?:\.babelrc|babel\.config)/.test((0, _nodePath.basename)(file)));
        for (const filename of files) {
          const fileHandling = await (0, _util.getBabelFileHandling)(filename);
          if (fileHandling === 'transpile') {
            await build(filename, posPath);
            numFilesCompiled++;
          }
        }
        for (const copy of copyable) {
          const relativeFn = (0, _nodePath.relative)(posPath, copy);
          const outEsm = (0, _nodePath.join)(outDir, relativeFn);
          await (0, _promises.mkdir)((0, _nodePath.dirname)(outEsm), {
            recursive: true
          });
          await (0, _promises.copyFile)(copy, outEsm);
          if (!noCjsDir) {
            const outCjs = (0, _nodePath.join)(cjsOutDir, relativeFn);
            await (0, _promises.mkdir)((0, _nodePath.dirname)(outCjs), {
              recursive: true
            });
            await (0, _promises.copyFile)(copy, outCjs);
          }
        }
      }
    }
    compileTime = _nodePerf_hooks.performance.now();
    if ((0, _nodeFs.existsSync)(outDir)) {
      tsStartTime = _nodePerf_hooks.performance.now();
      const files = (await (0, _util.getFiles)(outDir)).filter(file => dmctsRegex.test(file));
      for (const filename of files) {
        const base = (0, _nodePath.basename)(filename);
        let outCjs = (0, _nodePath.join)(noCjsDir ? outDir : cjsOutDir, base);
        if (keepFileExtension) {
          await (0, _promises.mkdir)((0, _nodePath.dirname)(outCjs), {
            recursive: true
          });
          await (0, _promises.copyFile)(filename, outCjs);
        } else {
          const {
            esm: esmTypes,
            cjs: cjsTypes
          } = await (0, _transform.updateDtsSpecifiers)(filename, outFileExtension);
          const dtsRegex = /(\.d\.ts)$/;
          const outEsmOrig = (0, _nodePath.join)(outDir, base);
          let outEsm = outEsmOrig;
          if (dtsRegex.test(base)) {
            const {
              esm,
              cjs
            } = outFileExtension;
            const esmExt = (0, _nodePath.extname)(esm);
            const cjsExt = (0, _nodePath.extname)(cjs);
            const esmOut = esmExt || esm;
            const cjsOut = cjsExt || cjs;
            const repl = cjsOut === '.cjs' ? '.d.cts' : '$1';
            outEsm = outEsm.replace(dtsRegex, `${esm.replace(esmOut, '')}$1`);
            outCjs = outCjs.replace(dtsRegex, `${cjs.replace(cjsOut, '')}${repl}`);
          }
          await (0, _promises.mkdir)((0, _nodePath.dirname)(outCjs), {
            recursive: true
          });
          await (0, _promises.writeFile)(outEsm, esmTypes);
          await (0, _promises.writeFile)(outCjs, cjsTypes);
          if (outEsmOrig !== outEsm) {
            await (0, _promises.rm)(outEsmOrig, {
              force: true
            });
          }
        }
        tsFilesUpdated++;
      }
      tsUpdateTime = _nodePerf_hooks.performance.now();
    }
    (0, _util.logResult)(`Successfully compiled ${numFilesCompiled} file${numFilesCompiled === 1 ? '' : 's'} as ESM and CJS in ${Math.abs(Math.round(compileTime - startTime))}ms with Babel ${_core.version}.`);
    if (tsFilesUpdated) {
      (0, _util.logResult)(`Successfully copied and updated ${tsFilesUpdated} TypeScript declaration file${tsFilesUpdated === 1 ? '' : 's'} in ${Math.abs(Math.round(tsUpdateTime - tsStartTime))}ms.`);
    }
  }
};
exports.babelDualPackage = babelDualPackage;
const url = new URL(import.meta.url);
const args = url.searchParams.get('args');
const realFileUrlArgv1 = await (0, _util.getRealPathAsFileUrl)(_nodeProcess.argv[1]);
if (parseFloat(_nodeProcess.versions.node) < 16.19) {
  (0, _util.logError)('This script requires a Node version >= 16.19.0');
} else if (import.meta.url === realFileUrlArgv1) {
  await babelDualPackage();
} else if (args) {
  let moduleArgs = null;
  try {
    moduleArgs = JSON.parse(args);
  } catch (err) {
    (0, _util.logError)('Invalid args passed in import query. Not parsable as JSON.');
    (0, _util.logResult)(`Error: ${err.message}`);
  }
  if (moduleArgs) {
    if (moduleArgs.files && Array.isArray(moduleArgs.files)) {
      let positionals = [];
      for (const file of moduleArgs.files) {
        positionals = positionals.concat(await (0, _glob.glob)(file, {
          ignore: 'node_modules/**'
        }));
      }
      delete moduleArgs.files;
      positionals = Object.entries(moduleArgs).concat(positionals);
      moduleArgs = positionals.flat().filter(Boolean);
      await babelDualPackage(moduleArgs);
    } else {
      (0, _util.logError)('No filenames found. Did you forget to pass a "files" key as an array of strings?');
    }
  }
}