#!/usr/bin/env node
import { cwd, argv, versions } from 'node:process';
import { performance } from 'node:perf_hooks';
import { resolve, dirname, relative, join, extname, basename } from 'node:path';
import { stat, writeFile, copyFile, mkdir, rm } from 'node:fs/promises';
import { existsSync } from 'node:fs';
import { version } from '@babel/core';
import { glob } from 'glob';
import { init } from './init.js';
import { transform, updateDtsSpecifiers } from './transform.js';
import { logError, logResult, getFiles, getOutExt, getEsmPlugins, getModulePresets, getRealPathAsFileUrl, getBabelFileHandling, addDefaultPresets } from './util.js';
const babelDualPackage = async moduleArgs => {
  const ctx = await init(moduleArgs, logError);
  if (ctx) {
    const {
      args,
      babelProjectConfig
    } = ctx;
    const {
      targets,
      plugins,
      presets,
      babelrc: unused,
      filename: overwritten,
      ...options
    } = babelProjectConfig.options;
    const outDir = resolve(relative(cwd(), args.values['out-dir']));
    const cjsOutDir = join(outDir, args.values['cjs-dir-name']);
    const keepFileExtension = args.values['keep-file-extension'];
    const outFileExtension = args.values['out-file-extension'];
    const noCjsDir = args.values['no-cjs-dir'];
    const minified = args.values.minified || (options.minified ?? false);
    const noComments = args.values['no-comments'] || (Object.hasOwn(options, 'comments') ? !options.comments : false);
    const sourceMaps = args.values['source-maps'] || (options.sourceMaps ?? false);
    const copyFiles = args.values['copy-files'];
    const {
      extensions
    } = args.values;
    if (!presets.length) {
      addDefaultPresets(presets, extensions);
    }
    let numFilesCompiled = 0;
    let compileTime = 0;
    let [tsFilesUpdated, tsUpdateTime, tsStartTime] = [0, 0, 0];
    const dmctsRegex = /\.d\.[mc]?ts$/;
    const esmPresets = getModulePresets(presets, 'esm');
    const cjsPresets = getModulePresets(presets, 'cjs');
    const esmPlugins = getEsmPlugins(plugins);
    const startTime = performance.now();
    const build = async (filename, positional) => {
      const {
        code,
        map
      } = await transform(filename, {
        ...options,
        targets,
        presets,
        plugins,
        filename,
        minified,
        sourceMaps,
        ast: false,
        sourceType: 'module',
        comments: !noComments,
        esmPresets,
        esmPlugins,
        cjsPresets,
        outFileExtension,
        keepFileExtension
      });
      if (code) {
        const extRegex = /(\.\w+)$/i;
        const relativeFn = positional ? relative(positional, filename) : basename(filename);
        const esmExt = getOutExt(filename, outFileExtension, keepFileExtension);
        const cjsExt = getOutExt(filename, outFileExtension, keepFileExtension, 'cjs');
        let outEsm = join(outDir, relativeFn);
        let outCjs = join(noCjsDir ? outDir : cjsOutDir, relativeFn);
        outEsm = outEsm.replace(extRegex, esmExt);
        outCjs = outCjs.replace(extRegex, cjsExt);
        await mkdir(dirname(outEsm), {
          recursive: true
        });
        await mkdir(dirname(outCjs), {
          recursive: true
        });
        await writeFile(outEsm, code.esm);
        await writeFile(outCjs, code.cjs);
        if (sourceMaps && map) {
          await writeFile(`${outEsm}.map`, JSON.stringify(map.esm, null, 2));
          await writeFile(`${outCjs}.map`, JSON.stringify(map.cjs, null, 2));
        }
      }
    };
    for (const pos of args.positionals) {
      const posPath = resolve(relative(cwd(), pos));
      let stats = null;
      try {
        stats = await stat(posPath);
      } catch {
        continue;
      }
      if (stats.isFile()) {
        const fileHandling = await getBabelFileHandling(posPath);
        if (!dmctsRegex.test(posPath) && extensions.includes(extname(posPath)) && fileHandling === 'transpile') {
          await build(posPath);
          numFilesCompiled++;
        }
      } else {
        const allFiles = await getFiles(posPath);
        const fileHasExpectedExt = file => extensions.includes(extname(file));
        const files = allFiles.filter(file => fileHasExpectedExt(file) && !dmctsRegex.test(file));
        const copyable = !copyFiles ? [] : allFiles.filter(file => !fileHasExpectedExt(file) && !/^(?:\.babelrc|babel\.config)/.test(basename(file)));
        for (const filename of files) {
          const fileHandling = await getBabelFileHandling(filename);
          if (fileHandling === 'transpile') {
            await build(filename, posPath);
            numFilesCompiled++;
          }
        }
        for (const copy of copyable) {
          const relativeFn = relative(posPath, copy);
          const outEsm = join(outDir, relativeFn);
          await mkdir(dirname(outEsm), {
            recursive: true
          });
          await copyFile(copy, outEsm);
          if (!noCjsDir) {
            const outCjs = join(cjsOutDir, relativeFn);
            await mkdir(dirname(outCjs), {
              recursive: true
            });
            await copyFile(copy, outCjs);
          }
        }
      }
    }
    compileTime = performance.now();
    if (existsSync(outDir)) {
      tsStartTime = performance.now();
      const files = (await getFiles(outDir)).filter(file => dmctsRegex.test(file));
      for (const filename of files) {
        const base = basename(filename);
        let outCjs = join(noCjsDir ? outDir : cjsOutDir, base);
        if (keepFileExtension) {
          await mkdir(dirname(outCjs), {
            recursive: true
          });
          await copyFile(filename, outCjs);
        } else {
          const {
            esm: esmTypes,
            cjs: cjsTypes
          } = await updateDtsSpecifiers(filename, outFileExtension);
          const dtsRegex = /(\.d\.ts)$/;
          const outEsmOrig = join(outDir, base);
          let outEsm = outEsmOrig;
          if (dtsRegex.test(base)) {
            const {
              esm,
              cjs
            } = outFileExtension;
            const esmExt = extname(esm);
            const cjsExt = extname(cjs);
            const esmOut = esmExt || esm;
            const cjsOut = cjsExt || cjs;
            const repl = cjsOut === '.cjs' ? '.d.cts' : '$1';
            outEsm = outEsm.replace(dtsRegex, `${esm.replace(esmOut, '')}$1`);
            outCjs = outCjs.replace(dtsRegex, `${cjs.replace(cjsOut, '')}${repl}`);
          }
          await mkdir(dirname(outCjs), {
            recursive: true
          });
          await writeFile(outEsm, esmTypes);
          await writeFile(outCjs, cjsTypes);
          if (outEsmOrig !== outEsm) {
            await rm(outEsmOrig, {
              force: true
            });
          }
        }
        tsFilesUpdated++;
      }
      tsUpdateTime = performance.now();
    }
    logResult(`Successfully compiled ${numFilesCompiled} file${numFilesCompiled === 1 ? '' : 's'} as ESM and CJS in ${Math.abs(Math.round(compileTime - startTime))}ms with Babel ${version}.`);
    if (tsFilesUpdated) {
      logResult(`Successfully copied and updated ${tsFilesUpdated} TypeScript declaration file${tsFilesUpdated === 1 ? '' : 's'} in ${Math.abs(Math.round(tsUpdateTime - tsStartTime))}ms.`);
    }
  }
};
const url = new URL(import.meta.url);
const args = url.searchParams.get('args');
const realFileUrlArgv1 = await getRealPathAsFileUrl(argv[1]);
if (parseFloat(versions.node) < 16.19) {
  logError('This script requires a Node version >= 16.19.0');
} else if (import.meta.url === realFileUrlArgv1) {
  await babelDualPackage();
} else if (args) {
  let moduleArgs = null;
  try {
    moduleArgs = JSON.parse(args);
  } catch (err) {
    logError('Invalid args passed in import query. Not parsable as JSON.');
    logResult(`Error: ${err.message}`);
  }
  if (moduleArgs) {
    if (moduleArgs.files && Array.isArray(moduleArgs.files)) {
      let positionals = [];
      for (const file of moduleArgs.files) {
        positionals = positionals.concat(await glob(file, {
          ignore: 'node_modules/**'
        }));
      }
      delete moduleArgs.files;
      positionals = Object.entries(moduleArgs).concat(positionals);
      moduleArgs = positionals.flat().filter(Boolean);
      await babelDualPackage(moduleArgs);
    } else {
      logError('No filenames found. Did you forget to pass a "files" key as an array of strings?');
    }
  }
}
export { babelDualPackage };